
-- SurgeonGeneral by Brimmstone
-- Version 1.2

SurgeonGeneral = LibStub("AceAddon-3.0"):NewAddon("SurgeonGeneral", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceSerializer-3.0", "AceTimer-3.0")

local LTT=LibStub:GetLibrary("LibQTip-1.0");
local tooltip;
local MaxProtocolVersion = 2;

local options = {
	name = "SurgeonGeneral",
	handler = SurgeonGeneral,
    type = "group",
    args = {
		Reset = {
			type = "execute",
			name = "Reset",
			desc = "Reset Window Position",
			func = "ResetWindowPosition",
			order = 1,
		},
		Verbose = {
			type = "toggle",
			name = "Verbose",
			desc = "Toggles Verbose Mode",
			set = "ToggleVerbose",
			get = "GetVerbose",
			order = 3,
		},
		Condense = {
			type = "toggle",
			name = "Condense Assignments",
			desc = "Multiple healers with the same assignment are grouped",
			set = "ToggleCondense",
			get = "GetCondense",
			order = 2,
		},
		TooltipScale = {
			type = "range",
			name = "Tooltip Scale",
			desc = "Set scale of mouseover tooltip",
			min = 0.5,
			max = 1.5,
			step = 0.05,
			bigStep = 0.05,
			get = function() return SurgeonGeneral.db.profile.TooltipScale end,
			set = function(_,value)
					SurgeonGeneral.db.profile.TooltipScale = value
					tooltip:SetScale(SurgeonGeneral.db.profile.TooltipScale)
				  end,
		},
		SwapStyle = {
			type	= "select",
			name	= "Swap Style",
			desc	= "Swap Style",
			get		= function() return SurgeonGeneral.db.profile.SwapStyle end,
			set		= function( info, value ) SurgeonGeneral.db.profile.SwapStyle = value end,
			values	= {
				[1] = "Swap Names",
				[2] = "Swap Assignments",
			},
		},
		--[[LogToChat = {
			type = "toggle",
			name = "Log Assignments to Chat",
			desc = "Logs assignemnts when sent out via chat",
			set = "ToggleLogToChat",
			get = "GetLogToChat",
			order = 4,
		},]]--
	},
}

local defaults = {
	profile = {
		Verbose = false,
		Condense = true,
		TooltipScale = 1.0,
		SwapStyle = 1,
	},
}

local AssignmentDefaults = {
	global = {
		assignments = {},
	},
}

local Version = "4.0.3.1 (Beta)"

local VariablesLoaded

local MainTankTable = {};

local AutoFillInProgress
local AutoFillCurrentUnit
local AutoFillCurrentUnitClassE
local AutoFillCurrentUnitClassL
local AutoFillCurrentID
local AutoFillTaint
local AutoFillCurrentSlot
local AutoFillExistingList = {};
local AutoFillDiscoverList = {};
local AutoFillRemoveList = {};
local AutoFillAddList = {};
local AutoFillTimer;
local AutoFillSkipInspectList = {};

local RaidNameList = {"Karazhan","Gruul's Lair","Magtheridon's Lair","Serpentshrine Cavern","Tempest Keep","Mount Hyjal","Black Temple","Sunwell Plateau","Naxxramas","Obsidian Sanctum","Eye of Eternity","Vault of Archavon","Ulduar","Trial of the Crusader","Trial of the Grand Crusader","Onyxia's Lair","Icecrown Citadel","Ruby Sanctum","Baradin Hold","Bastion of Twilight","Blackwing Descent","Throne of the Four Winds"};
local BossNameList = {};
BossNameList.KZ = {"Trash","Attumen","Moroes","Maiden of Virtue","Opera Event","Nightbane","The Curator","Terestian Illhoof","Shade of Aran","Netherspite","Prince Malchezaar"}
BossNameList.GL = {"Trash","High King Maulgar","Gruul"}
BossNameList.ML = {"Trash","Magtheridon"}
BossNameList.SSC = {"Trash","Hydross the Unstable","The Lurker Below","Leotheras the Blind","Fathom Lord Karathress","Morogrim Tidewalker","Lasy Vashj"}
BossNameList.TK = {"Trash","Al'ar","Void Reaver","High Astromancer Solarian","Kael'thas Sunstrider"}
BossNameList.MH = {"Trash","Rage Winterchill","Anetheron","Kaz'rogal","Azgalor","Archimonde"}
BossNameList.BT = {"Trash","High Warlord Naj'entus","Supremus","Shade of Akama","Teron Gorefiend","Gurtogg Bloodboil","Reliquary of Souls","Mother Sharhaz","Illidari Council","Illidan Stormrage"};
BossNameList.SWP = {"Trash","Kalecgos","Brutallus","Felmyst","Eredar Twins","M'uru","Kil'jaeden"};
BossNameList.NAX = {"Trash","Anub'Rekhan","Grand Widow Faerlina","Maexxna","Noth the Plaguebringer","Heigan the Unclean","Loatheb","Instructor Razuvious","Gothic the Harvester","The Four Horsemen","Patchwerk","Grobbulus","Gluth","Thaddius","Sapphiron","Kel'Thuzad"};
BossNameList.OS = {"Trash","Shadron","Tenebron","Vesperon","Sartharion"};
BossNameList.EOE = {"Malygos"};
BossNameList.VoA = {"Trash","Archavon the Stone Watcher","Emalon the Storm Watcher","Koralon the Flame Watcher","Toravon the Ice Watcher"};
BossNameList.Ulduar = {"Trash","Flame Leviathan","Ignis the Furnace Master","Razorscale","XT-002 Deconstructor","The Iron Council","Kologarn","Auriaya","Hodir","Thorim","Freya","Mimiron","General Vezax","Yogg-Saron","Algalon the Observer"};
BossNameList.TotC = {"Beasts of Northrend","Lord Jaraxxus","Faction Champions","Twin Val'kyr","Anub'arak"};
BossNameList.TotGC = {"Beasts of Northrend","Lord Jaraxxus","Faction Champions","Twin Val'kyr","Anub'arak"};
BossNameList.Onyxia = {"Trash","Onyxia"};
BossNameList.IC = {"Trash","Lord Marrowgar","Lady Deathwhisper","Gunship Battle","Deathbringer Saurfang","Festergut","Rotface","Professor Putricide","Blood Prince Council","Queen Lana'thel","Valithria Dreamwalker","Sindragosa","The Lich King"};
BossNameList.RS = {"Trash","Saviana Ragefire","Baltharus the Warborn","General Zarithrian","Halion"};
BossNameList.BH = {"Trash","Argaloth"};
BossNameList.BoT = {"Trash","Halfus Wyrmbreaker","Valiona & Theralion","Twilight Ascendants","Cho'gall","Sinestra"};
BossNameList.BWD = {"Trash","Magmaw","Omnotron Defense System","Chimaeron","Atramedes","Maloriak","Nefarian"};
BossNameList.TotFW = {"Trash","Conclave of Wind","Al'Akir"};

function SurgeonGeneral:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SurgeonGeneralDB", defaults, "Default")
	self.db2 = LibStub("AceDB-3.0"):New("SurgeonGeneralAssignments",AssignmentDefaults)
		
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SurgeonGeneral", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SurgeonGeneral", "SurgeonGeneral");
    self:RegisterChatCommand("sg", "ChatCommand");
	
	hooksecurefunc("NotifyInspect", SurgeonGeneral_NotifyInspect);
end

function SurgeonGeneral:ChatCommand(input)
	local x
	if not input or input:trim() == "" then
		SurgeonGeneral:ShowAssignmentFrame();
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SurgeonGeneral, "sg", "SurgeonGeneral", input)
	end
end

function SurgeonGeneral:ShowAssignmentFrame()
	SurgeonGeneralConfigFrame:Show();
	if SurgeonGeneral.db2.global.DatabaseVersion ~= 2 then
		SurgeonGeneral:ConvertAssignmentDatabase();
	end
	_G["VersionTextLabel"]:SetText(Version);
	SurgeonGeneral.db2.global.assignments["Clipboard"] = nil;
	UIDropDownMenu_SetWidth(RaidNameDropdown,175);
	UIDropDownMenu_SetWidth(BossNameDropdown,175);
	for x = 1,10 do
		if _G["HealerClass"..x.."Texture"]:GetTexture() == nil then
			UpdateHealerClass(x);
			SurgeonGeneral:ArrangeClassDrags();
		end
	end
end

function SurgeonGeneral:ResetWindowPosition()
	SurgeonGeneralConfigFrame:ClearAllPoints();
	SurgeonGeneralConfigFrame:SetPoint("CENTER","UIParent");
end

function SurgeonGeneral:ToggleVerbose()
	self.db.profile.Verbose = not self.db.profile.Verbose
end

function SurgeonGeneral:GetVerbose()
	return self.db.profile.Verbose;
end

function SurgeonGeneral:ToggleCondense()
	self.db.profile.Condense = not self.db.profile.Condense
end

function SurgeonGeneral:GetCondense()
	return self.db.profile.Condense;
end

function SurgeonGeneral:ToggleLogToChat()
	self.db.profile.LogToChat = not self.db.profile.LogToChat
end

function SurgeonGeneral:GetLogToChat()
	return self.db.profile.LogToChat;
end

function SurgeonGeneral:CopyBossFromTarget()
	local name;
	if (UnitExists("target")) then
		if (UnitIsPlayer("target")) then
			return;
		else
			name,_ = UnitName("target");
			_G["BossName"]:SetText(name);
		end
	end
end

function SurgeonGeneral:OnLoad()
	AutoFillInProgress = 0;
	VariablesLoaded = false;
	local x;
	_G["Phase1Name"]:SetText("Phase1");
	_G["Phase2Name"]:SetText("Phase2");
	_G["Phase3Name"]:SetText("Phase3");
	_G["Phase4Name"]:SetText("Phase4");
	_G["Phase5Name"]:SetText("Phase5");
	SurgeonGeneral:RegisterEvent("INSPECT_READY","AutoFillInspectReady");
	SurgeonGeneral:RegisterEvent("VARIABLES_LOADED","OnEvent");
	SurgeonGeneral:RegisterEvent("CHAT_MSG_ADDON");
	_G["AutoFillButton"]:SetScript("OnClick",self.AutoFillStart);
	_G["ReportButton"]:SetScript("OnClick",self.StartReport);
	_G["CopyAssignmentButton"]:SetScript("OnClick",self.CopyAssignments);
	_G["PasteAssignmentButton"]:SetScript("OnClick",self.PasteAssignments);
	_G["SaveAssignmentButton"]:SetScript("OnClick",self.SaveAssignments);
	--_G["AutoFillBossButton"]:SetScript("OnClick",self.CopyBossFromTarget);
	tooltip = LTT:Acquire("SurgeonGeneralTT",3,"LEFT","CENTER","RIGHT");
end

function SurgeonGeneral:OnEvent(event)
	if(event == "VARIABLES_LOADED") then
		VariablesLoaded = true;
	end
end


function SurgeonGeneral:CHAT_MSG_ADDON(arg1, prefix, msg, distro, author)
	if ((prefix == "CTRA") or (prefix == "oRA")) and (distro == "RAID") then
		-- Check to see if message is from Raid Leader or Raid Officer.
		local j,k,name,rank;
		for k = 1,GetNumRaidMembers() do
			name, rank = GetRaidRosterInfo(k);
			SurgeonGeneral:VerbosePrint(name..", "..author..", "..rank.." <end>");
			if (name == author) and (rank > 0) then
				-- Message from Raid Leader/Officer
				SurgeonGeneral:Parse_oRA2(author,strsplit("#",msg));
			end
		end
	elseif (prefix == "SGLC") then
		local Success, ProtocolVersion, MessageType, Data;
		Success, ProtocolVersion, MessageType, Data = SurgeonGeneral:Deserialize(msg);
		SurgeonGeneral:Parse_SGLC(ProtocolVersion, MessageType, author, SurgeonGeneral:Deserialize(Data));
	end
end

function SurgeonGeneral:Parse_SGLC(Version, MsgType, author, ...)  -- "..." arg is data
	SurgeonGeneral:VerbosePrint("SGLC Response :" .. Version .. ";" .. MsgType .. ";" .. author);
	if Version <= MaxProtocolVersion then
		if MsgType == "RoleResponse" then
			-- Check to make sure that we are autofilling, that they report as a healer, and that they're not already in the list
			if (AutoFillInProgress == 1) and (select(2,...) == "Healer") and (AutoFillDiscoverList[author] ~= true) then
				AutoFillDiscoverList[author] = true;
				AutoFillSkipInspectList[author] = true;
				SurgeonGeneral:ForcePrint("RoleResponse (Healer) from " .. author);
				AutoFillCurrentSlot = AutoFillCurrentSlot + 1;
			elseif (AutoFillInProgress == 1) and (select(2,...) == "NonHealer") then
				AutoFillSkipInspectList[author] = true;
				SurgeonGeneral:ForcePrint("RoleResponse (NonHealer) from " .. author);
			end
		end
	end
end

--[[
function SurgeonGeneral:CHAT_MSG_ADDON(arg1, prefix, msg, type, author)
	if (prefix ~= "CTRA") and (prefix ~= "oRA") then
		return
	end
	if (type ~= "RAID") then
		return
	end
	SurgeonGeneral:VerbosePrint(prefix..": "..msg..", "..type..", "..author.." <end>");
	
	-- Check to see if message is from Raid Leader or Raid Officer.
	local j,k,name,rank;
	for k = 1,GetNumRaidMembers() do
		name, rank = GetRaidRosterInfo(k);
		SurgeonGeneral:VerbosePrint(name..", "..author..", "..rank.." <end>");
		if (name == author) and (rank > 0) then
			-- Message from Raid Leader/Officer
			SurgeonGeneral:Parse_oRA2(author,strsplit("#",msg));
		end
	end
end

]]--

function SurgeonGeneral:Parse_oRA2(author,...)
	local i, num, name, msg;
	
	for i=1,select("#",...) do
		msg = select(i,...);
		
		num, name = select(3, msg:find("^SET (%d+) (.+)$"))
		if num and name then
			MainTankTable[tonumber(num)] = name;
			SurgeonGeneral:VerbosePrint("Added "..name.." as MainTank "..tonumber(num));
			return;
		end
				
		name = select(3, msg:find("^R (.+)$"))
		if name then
			for j=1,10 do
				if MainTankTable[j] == name then
					MainTankTable[j] = nil;
					SurgeonGeneral:VerbosePrint("Removed "..name.." as MainTank "..j);
				end
			end
		end
	end
end

function SetVar()
	--SurgeonGeneral:Print("Set Var");
end


function SurgeonGeneral:AssignmentScroll(frame,arg1)
	local k,WorkString;
	WorkString = frame:GetText();

	if arg1 == 1 then  --mousewheel up
		for k=10,1,-1 do
			WorkString = string.gsub(WorkString,"%%MT(%d+)",function(a) if tonumber(a)==k then return "%MT"..(k+1) end end)
		end
		WorkString = string.gsub(WorkString,"%%MT11","%%MT1")
	elseif arg1 == -1 then
		for k=1,10 do
			WorkString = string.gsub(WorkString,"%%MT(%d+)",function(a) if tonumber(a)==k then return "%MT"..(k-1) end end)
		end
		WorkString = string.gsub(WorkString,"%%MT0","%%MT10")
	end
	frame:SetText(WorkString);
end

function CalcPhases()
	local NumberOfPhases = 1;
	local k;
	k = 2;
	while k < 6 do
		if _G["Phase"..k.."Enable"]:GetChecked() then
			NumberOfPhases = k;
			k = k + 1;
		else
			k = 6;
		end
	end
	if NumberOfPhases < 4 then
		_G["Phase5Enable"]:Disable();
	else
		_G["Phase5Enable"]:Enable();
	end
	
	if NumberOfPhases < 3 then
		_G["Phase4Enable"]:Disable();
	else
		_G["Phase4Enable"]:Enable();
	end
	
	if NumberOfPhases < 2 then
		_G["Phase3Enable"]:Disable();
	else
		_G["Phase3Enable"]:Enable();
	end
	
	return NumberOfPhases;
end

function PhaseNameChanged()
	-- placeholder
end

function ValidatePhaseNames()
	if (_G["Phase1Name"]:GetText() == "") then
		_G["Phase1Name"]:SetText("Phase1");
	end
	if (_G["Phase2Name"]:GetText() == "") then
		_G["Phase2Name"]:SetText("Phase2");
	end
	if (_G["Phase3Name"]:GetText() == "") then
		_G["Phase3Name"]:SetText("Phase3");
	end
	if (_G["Phase4Name"]:GetText() == "") then
		_G["Phase4Name"]:SetText("Phase4");
	end
	if (_G["Phase5Name"]:GetText() == "") then
		_G["Phase5Name"]:SetText("Phase5");
	end
end

function PhaseEnable_OnClick()
	CalcPhases();
end


function MT_Lookup(x)
	local name,icon;
	
	if oRA3 then
		local oRA3_Tanks = oRA3:GetSortedTanks()
		name = oRA3_Tanks[tonumber(x)];
	else
		name = MainTankTable[tonumber(x)];
	end
	
	if name ~= nil then
		if UnitExists(name) then
			icon = GetRaidTargetIndex(name)
			if icon then
				return name.." ({rt"..icon.."})";
			else
				return name;
			end
		end
		return name;
	else
		return "MT"..x;
	end
end

function MT_NameLookup(x)
	local name;
	
	if oRA3 then
		local oRA3_Tanks = oRA3:GetSortedTanks()
		name = oRA3_Tanks[tonumber(x)];
	else
		name = MainTankTable[tonumber(x)];
	end
	
	if name ~= nil then
		return name;
	else
		return "MT"..x;
	end
end

function SurgeonGeneral:ReportToChannel(ReportChannel)
	local HealerMessage;
	local HealerClass;
	local j,k;
	local ReportChannelNumber;
	local ReportChannelName;
	
	ValidatePhaseNames();
		
	if (ReportChannel == "RAID") then
		ReportChannelNumber = nil;
		ReportChannelName = "RAID"
	else	
		ReportChannelNumber,_ = GetChannelName(ReportChannel);
		ReportChannelName = "CHANNEL"
		if(ReportChannelNumber == 0) then
			return;
		end
	end
	
	--SendChatMessage(" ",ReportChannelName,nil,ReportChannelNumber);
	if (UIDropDownMenu_GetText(BossNameDropdown) ~= nil) then
		SendChatMessage("Healing Assignments: "..UIDropDownMenu_GetText(BossNameDropdown),ReportChannelName,nil,ReportChannelNumber);
	else
		SendChatMessage("Healing Assignments",ReportChannelName,nil,ReportChannelNumber);
	end
	
	local AssignmentTable = {};	
	
	if CalcPhases() > 1 then
		HealerMessage = "Healer:  ";
		HealerMessage = HealerMessage .. _G["Phase1Name"]:GetText();
		for j = 2,CalcPhases() do
			HealerMessage = HealerMessage .. " ~ " .. _G["Phase"..j.."Name"]:GetText();
		end
		SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
	end
	
	for k = 1,10 do
		if not (_G["HealerName"..k]:GetText() == "") then
			HealerMessage = ""
			HealerMessage = HealerMessage .. _G["Healer"..k.."Phase1"]:GetText();
			for j = 2,CalcPhases() do
				HealerMessage = HealerMessage .. " ~ " .. _G["Healer"..k.."Phase"..j]:GetText();
			end
			
			HealerMessage = string.gsub(HealerMessage,"%%MT(%d+)",MT_Lookup);
			if (not self.db.profile.Condense) then
				HealerMessage = _G["HealerName"..k]:GetText() .. ":  "..HealerMessage;
				SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
			else
				if AssignmentTable[HealerMessage] == nil then
					AssignmentTable[HealerMessage] = _G["HealerName"..k]:GetText();
				else
					AssignmentTable[HealerMessage] = AssignmentTable[HealerMessage]..", ".._G["HealerName"..k]:GetText();
				end
			end
		end
	end
	
	if (not self.db.profile.Condense) then
		return;
	end
	
	for index,value in pairs(AssignmentTable) do
		if (type(index) == "number") then
			index = tostring(index)
		elseif (type(index) == "boolean") then
			if (index) then
				index = "true"
			else
				index = "false"
			end
		end
		SendChatMessage(value..": "..index,ReportChannelName,nil,ReportChannelNumber);
	end
end

function AssignmentTx(Field)
	return string.gsub(_G[Field]:GetText(),"%%MT(%d+)",MT_NameLookup)
end

function SurgeonGeneral:StartReport()
	local AssignmentsLogged,BossName;
	if (UIDropDownMenu_GetText(RaidNameDropdown) ~= nil) then
		BossName = UIDropDownMenu_GetText(RaidNameDropdown).." - "..UIDropDownMenu_GetText(BossNameDropdown);
	else
		BossName = "Assignments";
	end
	local CommMessageTx = SurgeonGeneral:Serialize(BossName,CalcPhases(),
									_G["HealerName1"]:GetText(), _G["HealerName2"]:GetText(),_G["HealerName3"]:GetText(),_G["HealerName4"]:GetText(),_G["HealerName5"]:GetText(),
									_G["HealerName6"]:GetText(), _G["HealerName7"]:GetText(),_G["HealerName8"]:GetText(),_G["HealerName9"]:GetText(),_G["HealerName10"]:GetText(),
									_G["Phase1Name"]:GetText(), _G["Phase2Name"]:GetText(),_G["Phase3Name"]:GetText(),_G["Phase4Name"]:GetText(),_G["Phase5Name"]:GetText(),
									AssignmentTx("Healer1Phase1"),AssignmentTx("Healer1Phase2"),AssignmentTx("Healer1Phase3"),AssignmentTx("Healer1Phase4"),AssignmentTx("Healer1Phase5"),
									AssignmentTx("Healer2Phase1"),AssignmentTx("Healer2Phase2"),AssignmentTx("Healer2Phase3"),AssignmentTx("Healer2Phase4"),AssignmentTx("Healer2Phase5"),
									AssignmentTx("Healer3Phase1"),AssignmentTx("Healer3Phase2"),AssignmentTx("Healer3Phase3"),AssignmentTx("Healer3Phase4"),AssignmentTx("Healer3Phase5"),
									AssignmentTx("Healer4Phase1"),AssignmentTx("Healer4Phase2"),AssignmentTx("Healer4Phase3"),AssignmentTx("Healer4Phase4"),AssignmentTx("Healer4Phase5"),
									AssignmentTx("Healer5Phase1"),AssignmentTx("Healer5Phase2"),AssignmentTx("Healer5Phase3"),AssignmentTx("Healer5Phase4"),AssignmentTx("Healer5Phase5"),
									AssignmentTx("Healer6Phase1"),AssignmentTx("Healer6Phase2"),AssignmentTx("Healer6Phase3"),AssignmentTx("Healer6Phase4"),AssignmentTx("Healer6Phase5"),
									AssignmentTx("Healer7Phase1"),AssignmentTx("Healer7Phase2"),AssignmentTx("Healer7Phase3"),AssignmentTx("Healer7Phase4"),AssignmentTx("Healer7Phase5"),
									AssignmentTx("Healer8Phase1"),AssignmentTx("Healer8Phase2"),AssignmentTx("Healer8Phase3"),AssignmentTx("Healer8Phase4"),AssignmentTx("Healer8Phase5"),
									AssignmentTx("Healer9Phase1"),AssignmentTx("Healer9Phase2"),AssignmentTx("Healer9Phase3"),AssignmentTx("Healer9Phase4"),AssignmentTx("Healer9Phase5"),
									AssignmentTx("Healer10Phase1"),AssignmentTx("Healer10Phase2"),AssignmentTx("Healer10Phase3"),AssignmentTx("Healer10Phase4"),AssignmentTx("Healer10Phase5"));
	
	CommMessageTx = SurgeonGeneral:Serialize(2,"Assignments",CommMessageTx);
	
	SurgeonGeneral:SendCommMessage("SGLC",CommMessageTx,"RAID",nil);
	
	if _G["RaidChannelEnable"]:GetChecked() then
		SurgeonGeneral:ReportToChannel("RAID");
	end
	
	if _G["CustomChannelEnable"]:GetChecked() then
		SurgeonGeneral:ReportToChannel(_G["CustomChannelName"]:GetText());
	end
end

function SurgeonGeneral:VerbosePrint(msg,color)
	if (self.db.profile.Verbose) then
		if color ~= nil then
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
		end
	end
end

function SurgeonGeneral:ForcePrint(msg,color)
	if color ~= nil then
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
	else
		DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
	end
end

function SurgeonGeneral:AutoFillStart()
	--[[
	local arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9;
	arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9 = GetTalentTabInfo(1,false,false,1);
	SurgeonGeneral:ForcePrint("TalentTab = " .. arg1 .. "," .. arg2 .. "," .. arg3 .. "," .. arg4 .. "," .. arg5 .. "," .. arg6 .. "," .. arg7 .. "," .. tostring(arg8) .. ",");
	]]--
	if (not UnitInRaid("player")) then
		SurgeonGeneral:ForcePrint("You are not in a raid","ffff00");
		AutoFillInProgress = 0;
		return;
	end
	
	SurgeonGeneral:ForcePrint("Auto Fill Starting...");
	
	wipe(AutoFillExistingList);
	wipe(AutoFillDiscoverList);
	wipe(AutoFillSkipInspectList);
	local x, Name;
	
	for x = 1,10 do
		Name = _G["HealerName"..x]:GetText();
		if (Name ~= "") then
			AutoFillExistingList[Name] = true;
		end
	end
	
	AutoFillInProgress = 1;
	AutoFillCurrentID = 0;
	AutoFillCurrentSlot = 1;
	
	local CommMessageTx = SurgeonGeneral:Serialize(2,"RoleRequest",SurgeonGeneral:Serialize(nil));
	SurgeonGeneral:SendCommMessage("SGLC",CommMessageTx,"RAID",nil);
	
	SurgeonGeneral:CancelTimer(AutoFillTimer,true);
	AutoFillTimer = SurgeonGeneral:ScheduleRepeatingTimer("AutoFillNext",2.5);
	--SurgeonGeneral:AutoFillNext();	
end

function SurgeonGeneral:AutoFillNext()
	if (AutoFillInProgress == 1) then
		while (AutoFillCurrentID < GetNumRaidMembers()) do
		
			-- break function if Autofill terminated while in this loop.
			if (AutoFillInProgress == 0) then
				return;
			end
			
			-- If somebody is already in the list (likely because they responded to the RoleRequest via SGLC) don't waste time inspecting them.
			repeat
				AutoFillCurrentID = AutoFillCurrentID + 1;
				AutoFillCurrentUnit = "raid"..AutoFillCurrentID;
			until (AutoFillSkipInspectList[UnitName(AutoFillCurrentUnit)] ~= true)
			
			if (AutoFillCurrentID >= GetNumRaidMembers()) then
				SurgeonGeneral:AutoFillStop();
				return;
			end
			
			AutoFillCurrentUnitClassL,AutoFillCurrentUnitClassE = UnitClass(AutoFillCurrentUnit);			
			if ((AutoFillCurrentUnitClassE == "WARRIOR") or (AutoFillCurrentUnitClassE == "ROGUE") or (AutoFillCurrentUnitClassE == "HUNTER") or (AutoFillCurrentUnitClassE == "MAGE") or (AutoFillCurrentUnitClassE == "WARLOCK") or (AutoFillCurrentUnitClassE == "DEATHKNIGHT")) then
				-- If it's not a healing class, go to next unit
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is not a healing class.");
			elseif UnitIsUnit("player",AutoFillCurrentUnit) then
				-- If it's me, just call the function to get my info (You can't inspect yourself).
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is me.");
				SurgeonGeneral:AutoFillGetTalents();
			elseif (CheckInteractDistance(AutoFillCurrentUnit,1)) then
				-- If it's somebody else and they're within inspect range, get their info.
				SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is a "..AutoFillCurrentUnitClassL..".  Inspecting...");
				NotifyInspect(AutoFillCurrentUnit);
				return;
			else
				-- If they're not within range, just skip them and move along
				SurgeonGeneral:ForcePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." was out of inspection range.");
			end
		end
		if (AutoFillInProgress == 1) then
			SurgeonGeneral:AutoFillStop();
		end
	end
end

function SurgeonGeneral:AutoFillStop()
	AutoFillInProgress = 0;
	SurgeonGeneral:CancelTimer(AutoFillTimer);
	SurgeonGeneral:ForcePrint("Auto Fill Sorting");
	
	
	-- Thanks Jerry for the table comparison help.
	local j,k,v;
	
	wipe(AutoFillRemoveList);
	wipe(AutoFillAddList);
	
	for v in pairs(AutoFillExistingList) do
		if not AutoFillDiscoverList[v] then
			AutoFillRemoveList[v] = true;
		end
	end
	
	for v in pairs(AutoFillDiscoverList) do
		if not AutoFillExistingList[v] then
			AutoFillAddList[#AutoFillAddList + 1] = v;
		end
	end
	
	-- Remove old names from UI
	for j = 1,10 do
		if (AutoFillRemoveList[_G["HealerName"..j]:GetText()]) then
			_G["HealerName"..j]:SetText("");
		end
	end
	
	j = 1;
	k = 1;
	while (j <= 10) do
		if (k > #AutoFillAddList) then
			j = 11;
		elseif (_G["HealerName"..j]:GetText() == "") then
			_G["HealerName"..j]:SetText(AutoFillAddList[k]);
			k = k + 1;
		end
		j = j + 1;
	end
	
	wipe(AutoFillExistingList);
	wipe(AutoFillDiscoverList);
	SurgeonGeneral:ForcePrint("Auto Fill Complete");
end

function SurgeonGeneral_NotifyInspect(unit)
	if (AutoFillInProgress == 0) then
		return;
	elseif (unit == AutoFillCurrentUnit) then
		AutoFillTaint = 0;
	else
		AutoFillTaint = 1;
		SurgeonGeneral:VerbosePrint("Inspect Taint!","ff0000");
		NotifyInspect(AutoFillCurrentUnit);
	end
end

function SurgeonGeneral:AutoFillGetTalents()
	local HealingTalents;
	local HealingTree;
	local HealingTreeName;
	local InspectFlag;
	local TalentThreshold = 31;
		
	if (AutoFillInProgress == 0) then
		return;
	elseif (AutoFillTaint == 1) then
		return;
	end
	
	if (UnitIsUnit("player",AutoFillCurrentUnit)) then
		InspectFlag = false;
	else
		InspectFlag = true;
	end
	
	local ActiveSpec = GetActiveTalentGroup(InspectFlag);
	
	-- Get number of healing talents/tree name.
	if (AutoFillCurrentUnitClassE == "PALADIN") then
		_,HealingTreeName,_,_,HealingTalents,_ = GetTalentTabInfo(1,InspectFlag,false,ActiveSpec);
	elseif (AutoFillCurrentUnitClassE == "PRIEST") then
		_,HealingTreeName,_,_,HealingTalents,_ = GetTalentTabInfo(2,InspectFlag,false,ActiveSpec);
		-- if Holy tree doesn't meet threshold, check for Disc healer.
		-- Thanks for P5hema2 from Curse for their user feedback.
		if HealingTalents < TalentThreshold then
			_,HealingTreeName,_,_,HealingTalents,_ = GetTalentTabInfo(1,InspectFlag,false,ActiveSpec);
		end
	elseif (AutoFillCurrentUnitClassE == "SHAMAN") then
		_,HealingTreeName,_,_,HealingTalents,_ = GetTalentTabInfo(3,InspectFlag,false,ActiveSpec);
	elseif (AutoFillCurrentUnitClassE == "DRUID") then
		_,HealingTreeName,_,_,HealingTalents,_ = GetTalentTabInfo(3,InspectFlag,false,ActiveSpec);
	else
		--  Not a healing class... should not have been inspected, but trapping anyways.
		return;
	end
	
	-- If you're a healer, you need 31 talents in your healing tree.
	SurgeonGeneral:VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." has "..HealingTalents.." in the "..HealingTreeName.." tree. (Spec:"..ActiveSpec..")");
	if (HealingTalents >= TalentThreshold) then
		local HealerName,_ = UnitName(AutoFillCurrentUnit);
		AutoFillDiscoverList[HealerName] = true;
		--_G["HealerName"..AutoFillCurrentSlot]:SetText(HealerName);
		AutoFillCurrentSlot = AutoFillCurrentSlot + 1;
	end
	
	if ((AutoFillCurrentSlot < 11) and (AutoFillCurrentID < GetNumRaidMembers())) then
		--SurgeonGeneral:AutoFillNext();
	else
		SurgeonGeneral:AutoFillStop();
	end
end

function SurgeonGeneral:AutoFillInspectReady()
	SurgeonGeneral:AutoFillGetTalents();
end

function HealerNameChange(healer)
	local id = tonumber(string.sub(healer:GetName(),11))
	UpdateHealerClass(id);
end

function UpdateHealerClass(icon)
	local _,englishClass = UnitClass(_G["HealerName"..icon]:GetText());
	if englishClass ~= nil then
		_G["HealerClass"..icon.."Texture"]:SetTexture("Interface\\Addons\\SurgeonGeneral\\Icons\\"..englishClass);
		_G["HealerClass"..icon.."DragTexture"]:SetTexture("Interface\\Addons\\SurgeonGeneral\\Icons\\"..englishClass);
	else
		_G["HealerClass"..icon.."Texture"]:SetTexture(nil);
		_G["HealerClass"..icon.."DragTexture"]:SetTexture(nil);
	end
end

function EditBox_OnEscapePressed()
	SurgeonGeneralConfigFrame:Hide()
end

function SurgeonGeneral:SaveToFile(Location)
	if type(SurgeonGeneral.db2.global.assignments) == nil then
		SurgeonGeneral.db2.global.assignments = {};
	end
	SurgeonGeneral.db2.global.assignments[Location] = 
	{_G["Phase1Name"]:GetText(), _G["Phase2Name"]:GetText(),_G["Phase3Name"]:GetText(),_G["Phase4Name"]:GetText(),_G["Phase5Name"]:GetText(),
	 _G["Healer1Phase1"]:GetText(),_G["Healer1Phase2"]:GetText(),_G["Healer1Phase3"]:GetText(),_G["Healer1Phase4"]:GetText(),_G["Healer1Phase5"]:GetText(),
	 _G["Healer2Phase1"]:GetText(),_G["Healer2Phase2"]:GetText(),_G["Healer2Phase3"]:GetText(),_G["Healer2Phase4"]:GetText(),_G["Healer2Phase5"]:GetText(),
	 _G["Healer3Phase1"]:GetText(),_G["Healer3Phase2"]:GetText(),_G["Healer3Phase3"]:GetText(),_G["Healer3Phase4"]:GetText(),_G["Healer3Phase5"]:GetText(),
	 _G["Healer4Phase1"]:GetText(),_G["Healer4Phase2"]:GetText(),_G["Healer4Phase3"]:GetText(),_G["Healer4Phase4"]:GetText(),_G["Healer4Phase5"]:GetText(),
	 _G["Healer5Phase1"]:GetText(),_G["Healer5Phase2"]:GetText(),_G["Healer5Phase3"]:GetText(),_G["Healer5Phase4"]:GetText(),_G["Healer5Phase5"]:GetText(),
	 _G["Healer6Phase1"]:GetText(),_G["Healer6Phase2"]:GetText(),_G["Healer6Phase3"]:GetText(),_G["Healer6Phase4"]:GetText(),_G["Healer6Phase5"]:GetText(),
	 _G["Healer7Phase1"]:GetText(),_G["Healer7Phase2"]:GetText(),_G["Healer7Phase3"]:GetText(),_G["Healer7Phase4"]:GetText(),_G["Healer7Phase5"]:GetText(),
	 _G["Healer8Phase1"]:GetText(),_G["Healer8Phase2"]:GetText(),_G["Healer8Phase3"]:GetText(),_G["Healer8Phase4"]:GetText(),_G["Healer8Phase5"]:GetText(),
	 _G["Healer9Phase1"]:GetText(),_G["Healer9Phase2"]:GetText(),_G["Healer9Phase3"]:GetText(),_G["Healer9Phase4"]:GetText(),_G["Healer9Phase5"]:GetText(),
	 _G["Healer10Phase1"]:GetText(),_G["Healer10Phase2"]:GetText(),_G["Healer10Phase3"]:GetText(),_G["Healer10Phase4"]:GetText(),_G["Healer10Phase5"]:GetText(),
	 _G["Phase2Enable"]:GetChecked(),_G["Phase3Enable"]:GetChecked(),_G["Phase4Enable"]:GetChecked(),_G["Phase5Enable"]:GetChecked()}
end

function SurgeonGeneral:LoadFromFile(Location)
	local x,y,z;
	if SurgeonGeneral.db2.global.assignments[Location] ~= nil then
		for x=1,5 do
			_G["Phase"..x.."Name"]:SetText(SurgeonGeneral.db2.global.assignments[Location][x]);
		end
		
		for x=1,10 do
			for y=1,5 do
				z = (x-1)*5+y+5;
				_G["Healer"..x.."Phase"..y]:SetText(SurgeonGeneral.db2.global.assignments[Location][z]);
			end
		end
		
		for x=2,5 do
			_G["Phase"..x.."Enable"]:SetChecked(SurgeonGeneral.db2.global.assignments[Location][x+54]);
		end
		CalcPhases();
	else
		SurgeonGeneral:ForcePrint("No data saved to "..Location)
	end
end



function SurgeonGeneral:SaveAssignments()
	SurgeonGeneral:SaveToFile(UIDropDownMenu_GetText(RaidNameDropdown).." - "..UIDropDownMenu_GetText(BossNameDropdown));
end

function SurgeonGeneral:CopyAssignments()
	SurgeonGeneral:SaveToFile("Clipboard");
end

function SurgeonGeneral:PasteAssignments()
	SurgeonGeneral:LoadFromFile("Clipboard");
end

function SurgeonGeneral:ConvertAssignmentDatabase()
	-- Converts assignments database to use both RaidName and BossName when saving.
	local x,y,z;
	local List= {};
	
	if SurgeonGeneral.db2.global.DatabaseVersion == 2 then
		return;
	end
	
	SurgeonGeneral:ForcePrint("Converting Assignment Database.");
	for x=1,getn(RaidNameList) do
		List = select(x,BossNameList.KZ, BossNameList.GL, BossNameList.ML, BossNameList.SSC, BossNameList.TK, BossNameList.MH, BossNameList.BT, BossNameList.SWP, BossNameList.NAX, BossNameList.OS, BossNameList.EOE, BossNameList.VoA, BossNameList.Ulduar, BossNameList.TotC, BossNameList.TotGC, BossNameList.Onyxia, BossNameList.IC, BossNameList.RS, BossNameList.BH, BossNameList.BoT, BossNameList.BWD, BossNameList.TotFW);
		for y=1,getn(List) do
			if SurgeonGeneral.db2.global.assignments[List[y]] ~= nil then
				SurgeonGeneral.db2.global.assignments[RaidNameList[x].." - "..List[y]] = SurgeonGeneral.db2.global.assignments[List[y]];
				SurgeonGeneral.db2.global.assignments[List[y]] = nil;
			end
		end
	end
	
	SurgeonGeneral.db2.global.DatabaseVersion = 2;
end


local info = {};

function RaidNameDropdown_OnLoad()
	if (VariablesLoaded == false) then
		return;
	end;
	
	local x;
	local List = RaidNameList;
	for x=1,getn(List) do
		info.text = List[x];
		info.value = x;
		info.owner = _G["RaidNameDropdown"]:GetParent();
		info.func = function() RaidName_OnSelect(x) end;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
end

function RaidName_OnSelect(value)
	if (UIDropDownMenu_GetSelectedValue(_G["RaidNameDropdown"]) ~= value) then
		UIDropDownMenu_SetSelectedValue(_G["RaidNameDropdown"], value);
		UIDropDownMenu_ClearAll(_G["BossNameDropdown"]);
		BossNameDropdown_OnLoad();
	end
end

function BossNameDropdown_OnLoad()
	if (VariablesLoaded == false) then
		return;
	end;
	
	local x;
	local List = {};
	
	if UIDropDownMenu_GetSelectedValue(_G["RaidNameDropdown"]) ~= nil then
		List = select(UIDropDownMenu_GetSelectedValue(_G["RaidNameDropdown"]),BossNameList.KZ, BossNameList.GL, BossNameList.ML, BossNameList.SSC, BossNameList.TK, BossNameList.MH, BossNameList.BT, BossNameList.SWP, BossNameList.NAX, BossNameList.OS, BossNameList.EOE, BossNameList.VoA, BossNameList.Ulduar, BossNameList.TotC, BossNameList.TotGC, BossNameList.Onyxia, BossNameList.IC, BossNameList.RS, BossNameList.BH, BossNameList.BoT, BossNameList.BWD, BossNameList.TotFW);
	end

	for x=1,getn(List) do
		info.text = List[x];
		info.value = x;
		info.owner = _G["BossNameDropdown"]:GetParent();
		info.hasarrow = true;
		info.func = function() BossName_OnSelect(x) end;
		info.checked = nil;
		UIDropDownMenu_AddButton(info);
	end
	
	if(UIDropDownMenu_GetSelectedValue(_G["BossNameDropdown"]) == nil) then
		UIDropDownMenu_SetSelectedValue(_G["BossNameDropdown"],1)
		UIDropDownMenu_SetText(_G["BossNameDropdown"],List[1])
		if _G["SurgeonGeneralConfigFrame"]:IsShown() then
			SurgeonGeneral:LoadFromFile(UIDropDownMenu_GetText(_G["RaidNameDropdown"]).." - "..UIDropDownMenu_GetText(_G["BossNameDropdown"]));
		end
	end
end

function BossName_OnSelect(value)
	UIDropDownMenu_SetSelectedValue(_G["BossNameDropdown"], value);
	SurgeonGeneral:LoadFromFile(UIDropDownMenu_GetText(_G["RaidNameDropdown"]).." - "..UIDropDownMenu_GetText(_G["BossNameDropdown"]));
end


function ClassColor(class)
	if class ~= nil then
		return string.format( "%2x%2x%2x", RAID_CLASS_COLORS[class].r * 255, RAID_CLASS_COLORS[class].g * 255, RAID_CLASS_COLORS[class].b * 255 )
	else
		return "7f7f7f";
	end
end

function ColorText(text,color)
	if color == nil then
		return text;
	else
		return "|cff"..color..text.."|r";
	end
end

function SurgeonGeneral:CreateTooltip()
	-- Columns: [Icon],HealerName,Phase1_Assignment, [... Phase5_Assignment]
	local NumColumns = CalcPhases() + 2;
	local t = {};
	local x,y;
	local temp,EnglishClass,LineColor,UnitTexture,HealerName;
	
	tooltip:Hide();
	tooltip:Clear();
	tooltip:SetScale(self.db.profile.TooltipScale);
	
	for x = 1,CalcPhases() do
		table.insert(t,"CENTER");
	end
	
	tooltip:SetColumnLayout(NumColumns,"LEFT","LEFT",unpack(t));
	tooltip:AddHeader("a");
	if (UIDropDownMenu_GetText(BossNameDropdown) ~= nil) then
		tooltip:SetCell(1,1,UIDropDownMenu_GetText(BossNameDropdown),"CENTER",NumColumns);
	else
		tooltip:SetCell(1,1,"SurgeonGeneral","CENTER",NumColumns);
	end
	
	tooltip:AddLine(" ");
	
	if(CalcPhases() > 1) then
		wipe(t)
		table.insert(t," ");
		table.insert(t," ");
		for x = 1,CalcPhases() do
			table.insert(t,_G["Phase"..x.."Name"]:GetText());
		end
		tooltip:AddLine(unpack(t));
	end
	
	
	for x = 1,10 do
		HealerName = _G["HealerName"..x]:GetText();
		if HealerName ~= "" then
			wipe(t);
			_,EnglishClass = UnitClass(HealerName);
			
			--For disconnected people....
			--UnitTexture = "|TInterface\\GossipFrame\\ActiveQuestIcon:0|t";
			
			if UnitIsDeadOrGhost(HealerName) then
				UnitTexture = "|TInterface\\Addons\\SurgeonGeneral\\Icons\\Tombstone:0|t";
			elseif UnitExists(HealerName) and UnitInRaid(HealerName) and (not UnitIsConnected(HealerName)) then
				UnitTexture = "|TInterface\\GossipFrame\\ActiveQuestIcon:0|t";
			else
				UnitTexture = "";
			end
			
			LineColor = ClassColor(EnglishClass);
			
			table.insert(t,UnitTexture);
			table.insert(t,ColorText(HealerName,LineColor));
			for y = 1,CalcPhases() do
				temp = _G["Healer"..x.."Phase"..y]:GetText();
				temp = string.gsub(temp,"%%MT(%d+)",MT_NameLookup)
				table.insert(t,ColorText(temp,LineColor));
			end
			tooltip:AddLine(unpack(t));
		end
	end
	
	tooltip:AddLine(" ");
	tooltip:AddLine("L");
	tooltip:SetCell(tooltip:GetLineCount(),1,"|cffff0000Left-Click|r to open setup window","LEFT",NumColumns)
	tooltip:AddLine("R");
	tooltip:SetCell(tooltip:GetLineCount(),1,"|cffff0000Right-Click|r to report assignments","LEFT",NumColumns)
	
	if oRA3 then
		tooltip:AddLine("oRA3");
		tooltip:SetCell(tooltip:GetLineCount(),1,"oRA3 Installed","LEFT",NumColumns)
	end
	
	
end


local LDB = LibStub("LibDataBroker-1.1", true)

local L_SG_LEFT = "Left click to open setup window"
local L_SG_RIGHT = "Right click to report assignments"
	local launcher = LDB:NewDataObject("SG", {
	type = "data source",
	text = "SG",
	OnClick = function(_, msg)
		if msg == "LeftButton" then
			if _G["SurgeonGeneralConfigFrame"]:IsShown() then
				_G["SurgeonGeneralConfigFrame"]:Hide()
			else
				SurgeonGeneral:ShowAssignmentFrame();
			end
		elseif msg == "RightButton" then
			SurgeonGeneral:StartReport();
		end
	end,
	icon = "Interface\\Icons\\Spell_ChargePositive",
	OnEnter = function(self)
		SurgeonGeneral:CreateTooltip();
		tooltip:SmartAnchorTo( self )
		tooltip:Show();
	end,
	OnLeave = function(self)
		tooltip:Hide();
	end,
	--[[OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("SurgeonGeneral")
		tooltip:AddLine(L_SG_LEFT)
		tooltip:AddLine(L_SG_RIGHT)
	end,]]--
})


function IsShiftkeyDown()
	return (IsLeftShiftkeyDown() or IsRightShiftKeyDown());
end

function SurgeonGeneral:ArrangeClassDrags()
	local x
	
	for x=1,10 do
		_G["HealerClass"..x.."Drag"]:SetAlpha(0.0);
		_G["HealerClass"..x.."Drag"]:ClearAllPoints();
		_G["HealerClass"..x.."Drag"]:SetPoint("CENTER","HealerClass"..x);
	end
end

function SurgeonGeneral:AssignmentSwap(x,y)
	local t,i;
	for i=1,5 do
		t = _G["Healer"..y.."Phase"..i]:GetText();
		_G["Healer"..y.."Phase"..i]:SetText(_G["Healer"..x.."Phase"..i]:GetText());
		_G["Healer"..x.."Phase"..i]:SetText(t);
	end
end

function SurgeonGeneral:HealerSwap(x,y)
	local t;
	t = _G["HealerName"..y]:GetText();
	_G["HealerName"..y]:SetText(_G["HealerName"..x]:GetText())
	_G["HealerName"..x]:SetText(t)
end

function SurgeonGeneral:IconOnClick()
	--SurgeonGeneral:ForcePrint("On Click");
end

function SurgeonGeneral:IconDragStart(frame,...)
	local id = tonumber(string.sub(frame:GetName(),12,-5))
	
	_G["HealerClass"..id.."Drag"]:SetAlpha(0.75);
	_G["HealerClass"..id.."Drag"]:SetFrameLevel(_G["HealerClass"..id]:GetFrameLevel()+1)
end

function SurgeonGeneral:IconDragStop(frame,...)
	local id = tonumber(string.sub(frame:GetName(),12,-5))
	local x;
	
	SurgeonGeneral:ArrangeClassDrags();
	
	if _G["HealerClass"..id.."Texture"]:GetTexture() == nil then
		return;
	end
	
	for x=1,10 do
		if MouseIsOver(_G["HealerClass"..x]) then
			if (SurgeonGeneral.db.profile.SwapStyle == 1) or IsShiftKeyDown() then
				SurgeonGeneral:HealerSwap(id,x);
			else
				SurgeonGeneral:AssignmentSwap(id,x);
			end
		end
	end
end
